package com.xyzc.start.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.domain.WebAppGroup;
import com.xyzc.start.domain.query.AppQuery;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.AppMapper;
import com.xyzc.start.mapper.CareerMapper;
import com.xyzc.start.mapper.IndustryMapper;
import com.xyzc.start.mapper.WebAppGroupMapper;
import com.xyzc.start.service.IAppPageService;
import jakarta.annotation.PostConstruct;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * 应用服务实现类
 */
@Service
@RequiredArgsConstructor
public class AppPageServiceImpl implements IAppPageService {

    private final IndustryMapper industryMapper;
    private final CareerMapper careerMapper;

    private final WebAppGroupMapper webAppGroupMapper;
    private final AppMapper appMapper;

    /**
     * 将所有的行业分类的和职业分类存储起来
     *
     * @return
     */
    private Map<Long, List<CareerVo>> industryMap;

    /**
     * 初始化数据
     */
    @PostConstruct
    public void init() {
        // 先获取所有的职业
        List<CareerVo> careerVos = careerMapper.selectVoList();

        // 按照行业id分组
        industryMap = careerVos.stream().collect(
                Collectors.groupingBy(CareerVo::getIndustryId)
        );
    }

    /**
     * 类型查询
     *
     * @return List<WebAppGroupVo>
     */
    @Override
    public List<WebAppGroupVo> queryByType() {

        return webAppGroupMapper.selectVoList(new LambdaQueryWrapper<WebAppGroup>()
                .orderByDesc(WebAppGroup::getSort));
    }

    /**
     * 行业查询
     *
     * @return
     */
    @Override
    public List<IndustryVo> queryByIndustry() {

        List<IndustryVo> industryVoList = industryMapper.selectVoList();
        // 将行业列表的id抽取出来
        List<Long> industryIds = industryVoList.stream().map(IndustryVo::getIndustryId).toList();

        List<CareerVo> careerVos = careerMapper.queryAppPageVoByIndustryIds(industryIds);

        //遍历行业列表，将职业列表通过行业id关联
        for (IndustryVo industryVo : industryVoList) {
            industryVo.setCareerList(careerVos.stream()
                    .filter(
                            careerVo -> careerVo.getIndustryId().equals(industryVo.getIndustryId())
                    ).toList()
            );
        }

        return industryVoList;
    }

    /**
     * 根据行业id查询所有所属的app列表
     *
     * @param careerId
     * @return
     */
    @Override
    public AppPageCareerVo queryByIndustryId(Long careerId, Long appId) {
        List<AppPageVo> appPageVos = careerMapper.queryAppPageVoByCareerId(SecurityUser.getUserId(), careerId, appId);
        AppPageCareerVo appPageCareerVo = new AppPageCareerVo();
        //为空跳出
        if (appPageVos == null || appPageVos.isEmpty()) {
            return null;
        }

        appPageCareerVo.setCareerName(appPageVos.get(0).getCareerName());
        appPageCareerVo.setAppPageList(appPageVos);

        // TODO 记得精选

        return appPageCareerVo;
    }

    /**
     * @param typeId
     * @return
     */
    @Override
    public WebAppGroupVo queryByTypeId(Long typeId, AppQuery appQuery) {
        List<AppPageVo> appVos = webAppGroupMapper.queryAppById(SecurityUser.getUserId(), typeId, appQuery);

        AssertUtils.isEmpty(appVos, ErrorCode.NO_SELECT_DATA);
        // 封装返回
        WebAppGroupVo webAppGroupVo = new WebAppGroupVo();
        webAppGroupVo.setAppPageList(appVos);
        webAppGroupVo.setGroupName(appVos.get(0).getAppGroupName());
        webAppGroupVo.setGroupId(appVos.get(0).getAppGroupId());

        return webAppGroupVo;
    }

    /**
     * app模糊查询
     *
     * @param appQuery
     * @return
     */
    @Override
    public List<BaseUserSelectAppVo> queryAppByQuery(AppQuery appQuery) {

        return appMapper.queryAppQuery(SecurityUser.getUserId(), appQuery);
    }
}
