package com.xyzc.start.utils;

import java.util.concurrent.atomic.AtomicLong;

public class SnowflakeIdWorker{
    // 起始时间戳（2020-01-01 00:00:00）
    private static final long START_TIMESTAMP = 1577808000000L;
    // 机器 ID 占用的位数
    private static final long WORKER_ID_BITS = 5L;
    // 数据中心 ID 占用的位数
    private static final long DATACENTER_ID_BITS = 5L;
    // 序列号占用的位数
    private static final long SEQUENCE_BITS = 1L;

    // 最大机器 ID
    private static final long MAX_WORKER_ID = -1L ^ (-1L << WORKER_ID_BITS);
    // 最大数据中心 ID
    private static final long MAX_DATACENTER_ID = -1L ^ (-1L << DATACENTER_ID_BITS);
    // 机器 ID 左移位数
    private static final long WORKER_ID_SHIFT = SEQUENCE_BITS;
    // 数据中心 ID 左移位数
    private static final long DATACENTER_ID_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS;
    // 时间戳左移位数
    private static final long TIMESTAMP_SHIFT = SEQUENCE_BITS + WORKER_ID_BITS + DATACENTER_ID_BITS;

    private final long workerId;
    private final long datacenterId;
    private final AtomicLong sequence = new AtomicLong(0L);
    private long lastTimestamp = -1L;

    public SnowflakeIdWorker(long workerId, long datacenterId) {
        if (workerId < 0 || workerId > MAX_WORKER_ID) {
            throw new IllegalArgumentException(String.format("Worker ID can't be greater than %d or less than 0", MAX_WORKER_ID));
        }
        if (datacenterId < 0 || datacenterId > MAX_DATACENTER_ID) {
            throw new IllegalArgumentException(String.format("DataCenter ID can't be greater than %d or less than 0", MAX_DATACENTER_ID));
        }
        this.workerId = workerId;
        this.datacenterId = datacenterId;
    }

    public synchronized long generateId() {
        long currentTimestamp = getCurrentTimestamp();

        if (currentTimestamp < lastTimestamp) {
            throw new RuntimeException("Clock moved backwards. Refusing to generate ID");
        }

        if (currentTimestamp == lastTimestamp) {
            sequence.incrementAndGet();
            if (sequence.get() >= (1 << SEQUENCE_BITS)) {
                currentTimestamp = waitNextMillis(currentTimestamp);
                sequence.set(0L);
            }
        } else {
            sequence.set(0L);
        }

        lastTimestamp = currentTimestamp;

        return ((currentTimestamp - START_TIMESTAMP) << TIMESTAMP_SHIFT)
                | (datacenterId << DATACENTER_ID_SHIFT)
                | (workerId << WORKER_ID_SHIFT)
                | sequence.get();
    }

    private long getCurrentTimestamp() {
        return System.currentTimeMillis();
    }

    private long waitNextMillis(long lastTimestamp) {
        long currentTimestamp = getCurrentTimestamp();
        while (currentTimestamp <= lastTimestamp) {
            currentTimestamp = getCurrentTimestamp();
        }
        return currentTimestamp;
    }

    public static void main(String[] args) {

        SnowflakeIdWorker idGenerator = new SnowflakeIdWorker(1, 1);
        for (int i = 0; i < 10; i++) {
            System.out.println(idGenerator.generateId());
        }
    }
}
